package com.fjhx.xmhjc.service.topic.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.fjhx.xmhjc.entity.column.dto.ColumnArticleDto;
import com.fjhx.xmhjc.entity.topic.po.TopicContent;
import com.fjhx.xmhjc.entity.website.po.WebsiteUsers;
import com.fjhx.xmhjc.mapper.topic.TopicContentMapper;
import com.fjhx.xmhjc.service.topic.TopicContentService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.xmhjc.service.website.WebsiteUsersService;
import com.fjhx.xmhjc.utils.WebsiteUserUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.wrapper.SqlField;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.xmhjc.entity.topic.vo.TopicContentVo;
import com.fjhx.xmhjc.entity.topic.dto.TopicContentSelectDto;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.fjhx.xmhjc.entity.topic.dto.TopicContentDto;
import cn.hutool.core.bean.BeanUtil;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 内容主题 服务实现类
 * </p>
 *
 * @author hj
 * @since 2023-11-20
 */
@Service
public class TopicContentServiceImpl extends ServiceImpl<TopicContentMapper, TopicContent> implements TopicContentService {
    @Resource
    private WebsiteUsersService websiteUsersService;


    @Override
    public List<TopicContentVo> getList(TopicContentSelectDto dto) {
        IWrapper<TopicContent> wrapper = getWrapper();
        wrapper.orderByDesc("tc", TopicContent::getId);
        List<TopicContentVo> list = this.baseMapper.getList(wrapper);
        return list;
    }

    @Override
    public Page<TopicContentVo> getPage(TopicContentSelectDto dto) {
        IWrapper<TopicContent> wrapper = getWrapper();

        if(StringUtils.isNotEmpty(dto.getToping())){
            wrapper.eq("tc", TopicContent::getToping,dto.getToping());
        }

        wrapper.orderByDesc("tc", TopicContent::getId);
        wrapper.keyword(dto, new SqlField("tc", TopicContent::getTitle));
        Page<TopicContentVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<TopicContentVo> records = page.getRecords();
        for (TopicContentVo record : records) {
            record.setAuthorName(websiteUsersService.getAuthorName(record.getAuthor()));
        }
        return page;
    }

    @Override
    public TopicContentVo detail(Long id) {
        TopicContent TopicContent = this.getById(id);
        TopicContentVo result = BeanUtil.toBean(TopicContent, TopicContentVo.class);
        return result;
    }

    @Override
    public void add(TopicContentDto topicContentDto) {
        this.save(topicContentDto);
    }

    @Override
    public void edit(TopicContentDto topicContentDto) {
        this.updateById(topicContentDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }


    @Override
    public Page<TopicContentVo> contentPage(TopicContentSelectDto dto, String sortType) {
        IWrapper<TopicContent> wrapper = getWrapper();
        if("1".equals(sortType)) {
            wrapper.orderByDesc("tc", TopicContent::getToping);
        }else if("3".equals(sortType)) {
            wrapper.eq("tc", TopicContent::getAuthor, WebsiteUserUtil.getLoginWebsiteUser().getId());
        }
        wrapper.orderByDesc("tc", TopicContent::getId);
        wrapper.eq(StrUtil.isNotBlank(dto.getType()), "tc.type", dto.getType());
        Page<TopicContentVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public void extracted(ColumnArticleDto columnArticleDto) {
        TopicContent topicContent = getById(columnArticleDto.getId());
        if (ObjectUtil.isNull(topicContent)){
            TopicContentDto topicContentDto = new TopicContentDto();
            topicContentDto.setType("2");
            topicContentDto.setId(columnArticleDto.getId());
            add(topicContentDto);
        }
    }
}
